@@ -1,13 +1,13 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2023 Mike Fährmann
+# Copyright 2015-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
 import sys
-from ..util import re_compile
+from ..text import re_compile
 
 modules = [
     "2ch",
@@ -40,9 +40,11 @@ modules = [
     "chevereto",
     "cien",
     "civitai",
+    "comick",
     "comicvine",
     "cyberdrop",
     "danbooru",
+    "dankefuerslesen",
     "desktopography",
     "deviantart",
     "discord",
@@ -63,6 +65,8 @@ modules = [
     "gelbooru",
     "gelbooru_v01",
     "gelbooru_v02",
+    "girlsreleased",
+    "girlswithmuscle",
     "gofile",
     "hatenablog",
     "hentai2read",
@@ -88,13 +92,14 @@ modules = [
     "issuu",
     "itaku",
     "itchio",
+    "iwara",
     "jschan",
     "kabeuchi",
     "keenspot",
-    "kemonoparty",
+    "kemono",
     "khinsider",
-    "koharu",
     "komikcast",
+    "leakgallery",
     "lensdump",
     "lexica",
     "lightroom",
@@ -102,19 +107,20 @@ modules = [
     "lofter",
     "luscious",
     "lynxchan",
+    "madokami",
     "mangadex",
     "mangafox",
     "mangahere",
     "manganelo",
     "mangapark",
     "mangaread",
-    "mangasee",
     "mangoxo",
     "misskey",
     "motherless",
     "myhentaigallery",
     "myportfolio",
-    "naver",
+    "naverblog",
+    "naverchzzk",
     "naverwebtoon",
     "nekohouse",
     "newgrounds",
@@ -123,6 +129,7 @@ modules = [
     "nitter",
     "nozomi",
     "nsfwalbum",
+    "nudostar",
     "paheal",
     "patreon",
     "pexels",
@@ -142,9 +149,11 @@ modules = [
     "pornhub",
     "pornpics",
     "postmill",
+    "rawkuma",
     "reactor",
     "readcomiconline",
     "realbooru",
+    "redbust",
     "reddit",
     "redgifs",
     "rule34us",
@@ -153,6 +162,7 @@ modules = [
     "saint",
     "sankaku",
     "sankakucomplex",
+    "schalenetwork",
     "scrolller",
     "seiga",
     "senmanga",
@@ -226,8 +236,7 @@ modules = [
 def find(url):
     """Find a suitable extractor for the given URL"""
     for cls in _list_classes():
-        match = cls.pattern.match(url)
-        if match:
+        if match := cls.pattern.match(url):
             return cls(match)
     return None
 
@@ -242,8 +251,7 @@ def add(cls):
 
 def add_module(module):
     """Add all extractors in 'module' to the list of available extractors"""
-    classes = _get_classes(module)
-    if classes:
+    if classes := _get_classes(module):
         if isinstance(classes[0].pattern, str):
             for cls in classes:
                 cls.pattern = re_compile(cls.pattern)
