@@ -18,8 +18,8 @@ class LynxchanExtractor(BaseExtractor):
 
 BASE_PATTERN = LynxchanExtractor.update({
     "bbw-chan": {
-        "root": "https://bbw-chan.nl",
-        "pattern": r"bbw-chan\.nl",
+        "root": "https://bbw-chan.link",
+        "pattern": r"bbw-chan\.(?:link|nl)",
     },
     "kohlchan": {
         "root": "https://kohlchan.net",
@@ -40,24 +40,17 @@ class LynxchanThreadExtractor(LynxchanExtractor):
     filename_fmt = "{postId}{num:?-//} {filename}.{extension}"
     archive_fmt = "{boardUri}_{postId}_{num}"
     pattern = BASE_PATTERN + r"/([^/?#]+)/res/(\d+)"
-    example = "https://bbw-chan.nl/a/res/12345.html"
-
-    def __init__(self, match):
-        LynxchanExtractor.__init__(self, match)
-        index = match.lastindex
-        self.board = match.group(index-1)
-        self.thread = match.group(index)
+    example = "https://endchan.org/a/res/12345.html"
 
     def items(self):
-        url = "{}/{}/res/{}.json".format(self.root, self.board, self.thread)
-        thread = self.request(url).json()
+        url = f"{self.root}/{self.groups[-2]}/res/{self.groups[-1]}.json"
+        thread = self.request_json(url)
         thread["postId"] = thread["threadId"]
         posts = thread.pop("posts", ())
 
         yield Message.Directory, thread
         for post in itertools.chain((thread,), posts):
-            files = post.pop("files", ())
-            if files:
+            if files := post.pop("files", ()):
                 thread.update(post)
                 for num, file in enumerate(files):
                     file.update(thread)
@@ -71,16 +64,12 @@ class LynxchanBoardExtractor(LynxchanExtractor):
     """Extractor for LynxChan boards"""
     subcategory = "board"
     pattern = BASE_PATTERN + r"/([^/?#]+)(?:/index|/catalog|/\d+|/?$)"
-    example = "https://bbw-chan.nl/a/"
-
-    def __init__(self, match):
-        LynxchanExtractor.__init__(self, match)
-        self.board = match.group(match.lastindex)
+    example = "https://endchan.org/a/"
 
     def items(self):
-        url = "{}/{}/catalog.json".format(self.root, self.board)
-        for thread in self.request(url).json():
-            url = "{}/{}/res/{}.html".format(
-                self.root, self.board, thread["threadId"])
+        board = self.groups[-1]
+        url = f"{self.root}/{board}/catalog.json"
+        for thread in self.request_json(url):
+            url = f"{self.root}/{board}/res/{thread['threadId']}.html"
             thread["_extractor"] = LynxchanThreadExtractor
             yield Message.Queue, url, thread
