@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2020-2023 Mike Fährmann
+# Copyright 2020-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -21,8 +21,7 @@ class ComparePP(PostProcessor):
             self._compare = self._compare_size
         self._equal_exc = self._equal_cnt = 0
 
-        equal = options.get("equal")
-        if equal:
+        if equal := options.get("equal"):
             equal, _, emax = equal.partition(":")
             self._equal_max = text.parse_int(emax)
             if equal == "abort":
@@ -62,12 +61,10 @@ class ComparePP(PostProcessor):
     def _compare(self, f1, f2):
         return self._compare_size(f1, f2) and self._compare_content(f1, f2)
 
-    @staticmethod
-    def _compare_size(f1, f2):
+    def _compare_size(self, f1, f2):
         return os.stat(f1).st_size == os.stat(f2).st_size
 
-    @staticmethod
-    def _compare_content(f1, f2):
+    def _compare_content(self, f1, f2):
         size = 16384
         with open(f1, "rb") as fp1, open(f2, "rb") as fp2:
             while True:
